package PoorBooru::API::V0::Controller::Media;
use v5.36;
use strict;
use warnings;

use Mojo::Base "Mojolicious::Controller";

my $MEDIA_SEARCH_OPTS = {
	order_by => { -desc => "media_id" },
	rows     => 5,
};

sub list ($self)
{
	my $v = $self->validation;
	my $page = $v->optional("page")->num(1, undef)->param // 1;

	return $self->render(
		json => {error => "Invalid page number"},
		status => 400,
	) if $v->has_error;

	my $paged_media = $self->schema->resultset("Media")
	    ->search({}, $MEDIA_SEARCH_OPTS)->page($page);
	my @media = map +{
		media_id      => $_->media_id,
		seaweedfs_fid => $_->seaweedfs_fid,
	}, $paged_media->all;

	return $self->render(json => {
		media => \@media,
		pager => { $self->pager($paged_media->pager) },
	});
};

sub show ($self)
{
	my $media_id = $self->stash("media_id");
	my $media = $self->schema->resultset("Media")
	    ->single({ media_id => $media_id });

	return $self->render(
		json   => {error => "Media not found"},
		status => 404,
	) if !defined($media);

	return $self->render(json => {
		id           => $media->media_id,
		name         => $media->filename,
		download     => $media->seaweedfs_fid,
		content_type => $media->content_type,
	});
};

1;
